@@ -1,4 +1,4 @@
-/* $Id: interface.c 1845 2007-04-27 02:48:10Z aturner $ */
+/* $Id: interface.c 1928 2007-10-26 17:21:35Z aturner $ */
 
 /*
  * Copyright (c) 2007 Aaron Turner.
@@ -40,7 +40,7 @@
 #include "common.h"
 #include "interface.h"
 
-/*
+/**
  * Method takes a user specified device name and returns
  * the canonical name for that device.  This allows me to 
  * create named interface aliases on platforms like Windows
@@ -75,6 +75,9 @@ get_interface(interface_list_t *list, const char *alias)
     return(NULL);
 }
 
+/** 
+ * Get all available interfaces as an interface_list *
+ */
 interface_list_t *
 get_interface_list(void)
 {
@@ -97,6 +100,10 @@ get_interface_list(void)
     list_ptr = list_head;
     
     while (pcap_if_ptr != NULL) {
+        if (i > 0) {
+            list_ptr->next = (interface_list_t *)safe_malloc(sizeof(interface_list_t));
+            list_ptr = list_ptr->next;
+        }
         strlcpy(list_ptr->name, pcap_if_ptr->name, sizeof(list_ptr->name));
         
         /* description is usually null under Unix */
@@ -106,14 +113,14 @@ get_interface_list(void)
         sprintf(list_ptr->alias, "%%%d", i++);
         list_ptr->flags = pcap_if_ptr->flags;
         pcap_if_ptr = pcap_if_ptr->next;
-        list_ptr->next = (interface_list_t *)safe_malloc(sizeof(interface_list_t));
-        list_ptr = list_ptr->next;
     }
-    free(list_ptr); /* free the last entry which was never used */
     pcap_freealldevs(pcap_if);
     return(list_head);
 }
 
+/**
+ * Prints all the available interfaces found by get_interface_list()
+ */
 void
 list_interfaces(interface_list_t *list)
 {
