@@ -1,4 +1,4 @@
-/* $Id: sendpacket.c 1804 2007-04-14 22:48:15Z aturner $ */
+/* $Id: sendpacket.c 1935 2007-11-01 16:46:28Z aturner $ */
 
 /*
  * Copyright (c) 2006 Aaron Turner.
@@ -145,19 +145,19 @@ static int get_iface_index(int fd, const int8_t *device, char *);
 #include <sys/uio.h>
 #include <net/if_dl.h> // used for get_hwaddr_bpf()
 
-static sendpacket_t *sendpacket_open_bpf(const char *, char *) __attribute__((unused));
-static struct tcpr_ether_addr *sendpacket_get_hwaddr_bpf(sendpacket_t *) __attribute__((unused));
+static sendpacket_t *sendpacket_open_bpf(const char *, char *) _U_;
+static struct tcpr_ether_addr *sendpacket_get_hwaddr_bpf(sendpacket_t *) _U_;
 
 #endif /* HAVE_BPF */
 
 #ifdef HAVE_LIBNET
-static sendpacket_t *sendpacket_open_libnet(const char *, char *) __attribute__((unused));
-static struct tcpr_ether_addr *sendpacket_get_hwaddr_libnet(sendpacket_t *) __attribute__((unused));
+static sendpacket_t *sendpacket_open_libnet(const char *, char *) _U_;
+static struct tcpr_ether_addr *sendpacket_get_hwaddr_libnet(sendpacket_t *) _U_;
 #endif /* HAVE_LIBNET */
 
 #if (defined HAVE_PCAP_INJECT || defined HAVE_PCAP_SENDPACKET)
-static sendpacket_t *sendpacket_open_pcap(const char *, char *) __attribute__((unused));
-static struct tcpr_ether_addr *sendpacket_get_hwaddr_pcap(sendpacket_t *) __attribute__((unused));
+static sendpacket_t *sendpacket_open_pcap(const char *, char *) _U_;
+static struct tcpr_ether_addr *sendpacket_get_hwaddr_pcap(sendpacket_t *) _U_;
 #endif /* HAVE_PCAP_INJECT || HAVE_PACKET_SENDPACKET */
 
 static void sendpacket_seterr(sendpacket_t *sp, const char *fmt, ...);
@@ -166,7 +166,7 @@ static void sendpacket_seterr(sendpacket_t *sp, const char *fmt, ...);
 extern volatile int didsig;
 
 
-/*
+/**
  * returns number of bytes sent on success or -1 on error
  * Note: it is theoretically possible to get a return code >0 and < len
  * which for most people would be considered an error (the packet wasn't fully sent)
@@ -266,7 +266,11 @@ TRY_SEND_AGAIN:
     return retcode;
 }
 
-
+/**
+ * Open the given network device name and returns a sendpacket_t struct
+ * pass the error buffer (in case there's a problem) and the direction
+ * that this interface represents
+ */
 sendpacket_t *
 sendpacket_open(const char *device, char *errbuf, tcpr_dir_t direction)
 {
@@ -291,7 +295,9 @@ sendpacket_open(const char *device, char *errbuf, tcpr_dir_t direction)
     return sp;
 }
 
-
+/**
+ * Get packet stats for the given sendpacket_t
+ */
 char *
 sendpacket_getstat(sendpacket_t *sp)
 {
@@ -309,16 +315,19 @@ sendpacket_getstat(sendpacket_t *sp)
     return(buf);
 }
 
+/**
+ * close the given sendpacket
+ */
 int
 sendpacket_close(sendpacket_t *sp)
 {
     assert(sp);
 
-    free(sp);
+    safe_free(sp);
     return 0;
 }
 
-/*
+/**
  * returns the Layer 2 address of the interface current 
  * open.  on error, return NULL
  */
@@ -344,7 +353,7 @@ sendpacket_get_hwaddr(sendpacket_t *sp)
     return addr;
 }
 
-/*
+/**
  * returns the error string
  */
 char *
@@ -354,7 +363,7 @@ sendpacket_geterr(sendpacket_t *sp)
     return sp->errbuf;
 }
 
-/*
+/**
  * Set's the error string
  */
 static void
@@ -374,15 +383,19 @@ sendpacket_seterr(sendpacket_t *sp, const char *fmt, ...)
 
 
 #if defined HAVE_PCAP_INJECT || defined HAVE_PCAP_SENDPACKET
+/**
+ * Inner sendpacket_open() method for using libpcap
+ */
 static sendpacket_t *
 sendpacket_open_pcap(const char *device, char *errbuf)
 {
     pcap_t *pcap;
     sendpacket_t *sp;
-/*
+#ifdef BIOCSHDRCMPLT
     u_int spoof_eth_src = 1;
     int fd;
- */
+#endif
+
     assert(device);
     assert(errbuf);
 
@@ -395,17 +408,22 @@ sendpacket_open_pcap(const char *device, char *errbuf)
     sp = (sendpacket_t *)safe_malloc(sizeof(sendpacket_t));
     strlcpy(sp->device, device, sizeof(sp->device));
     sp->handle.pcap = pcap;
-
-/*
-    fd = pcap_get_selectable_fd(pcap);
-    if (ioctl(fd, BIOCSHDRCMPLT, &spoof_eth_src) == -1) {
-        errx(1, "Unable to enable source MAC spoof support: %s", strerror(errno));
-    }
- */
     
+#ifdef BIOCSHDRCMPLT
+    /* 
+     * Only systems using BPF on the backend need this... 
+     * other systems don't have ioctl and will get compile errors.
+     */
+    fd = pcap_get_selectable_fd(pcap);
+    if (ioctl(fd, BIOCSHDRCMPLT, &spoof_eth_src) == -1)
+        errx(1, "Unable to enable source MAC spoof support: %s", strerror(errno));
+#endif
     return sp;
 }
 
+/**
+ * Get the hardware MAC address for the given interface using libpcap
+ */
 static struct tcpr_ether_addr *
 sendpacket_get_hwaddr_pcap(sendpacket_t *sp)
 {
@@ -416,6 +434,9 @@ sendpacket_get_hwaddr_pcap(sendpacket_t *sp)
 #endif /* HAVE_PCAP_INJECT || HAVE_PCAP_SENDPACKET */
 
 #if defined HAVE_LIBNET
+/**
+ * Inner sendpacket_open() method for using libnet
+ */
 static sendpacket_t * 
 sendpacket_open_libnet(const char *device, char *errbuf)
 {
@@ -436,6 +457,9 @@ sendpacket_open_libnet(const char *device, char *errbuf)
     return sp;    
 }
 
+/**
+ * Get the hardware MAC address for the given interface using libnet
+ */
 static struct tcpr_ether_addr *
 sendpacket_get_hwaddr_libnet(sendpacket_t *sp)
 {
@@ -455,6 +479,9 @@ sendpacket_get_hwaddr_libnet(sendpacket_t *sp)
 #endif /* HAVE_LIBNET */
 
 #if defined HAVE_PF_PACKET
+/**
+ * Inner sendpacket_open() method for using Linux's PF_PACKET
+ */
 static sendpacket_t *
 sendpacket_open_pf(const char *device, char *errbuf)
 {
@@ -554,7 +581,9 @@ sendpacket_open_pf(const char *device, char *errbuf)
     return sp;
 }
 
-/* get the interface index (necessary for sending packets w/ PF_PACKET) */
+/** 
+ * get the interface index (necessary for sending packets w/ PF_PACKET) 
+ */
 static int
 get_iface_index(int fd, const int8_t *device, char *errbuf) {
     struct ifreq ifr;
@@ -570,9 +599,8 @@ get_iface_index(int fd, const int8_t *device, char *errbuf) {
     return ifr.ifr_ifindex;
 }              
 
-/*
- * get's the hardware address via Linux's PF packet
- * interface
+/**
+ * get's the hardware address via Linux's PF packet interface
  */
 struct tcpr_ether_addr *
 sendpacket_get_hwaddr_pf(sendpacket_t *sp)
@@ -610,6 +638,9 @@ sendpacket_get_hwaddr_pf(sendpacket_t *sp)
 #endif /* HAVE_PF_PACKET */
 
 #if defined HAVE_BPF
+/**
+ * Inner sendpacket_open() method for using BSD's BPF interface
+ */
 static sendpacket_t *
 sendpacket_open_bpf(const char *device, char *errbuf)
 {
@@ -619,7 +650,7 @@ sendpacket_open_bpf(const char *device, char *errbuf)
     struct ifreq ifr;
     struct bpf_version bv;
     u_int v;
-#if defined(BIOCGHDRCMPLT) && defined(BIOCSHDRCMPLT) && !(__APPLE__)
+#if defined(BIOCGHDRCMPLT) && defined(BIOCSHDRCMPLT)
     u_int spoof_eth_src = 1;
 #endif
     
@@ -676,7 +707,7 @@ sendpacket_open_bpf(const char *device, char *errbuf)
      *  NetBSD and FreeBSD BPF have an ioctl for enabling/disabling
      *  automatic filling of the link level source address.
      */
-#if defined(BIOCGHDRCMPLT) && defined(BIOCSHDRCMPLT) && !(__APPLE__)
+#if defined(BIOCGHDRCMPLT) && defined(BIOCSHDRCMPLT)
     if (ioctl(mysocket, BIOCSHDRCMPLT, &spoof_eth_src) == -1) {
         snprintf(errbuf, SENDPACKET_ERRBUF_SIZE, 
             "Unable to enable spoofing src MAC: %s", strerror(errno));
@@ -725,6 +756,9 @@ sendpacket_open_bpf(const char *device, char *errbuf)
     return sp; 
 }
 
+/**
+ * Get the interface hardware MAC address when using BPF
+ */
 struct tcpr_ether_addr *
 sendpacket_get_hwaddr_bpf(sendpacket_t *sp)
 {
@@ -752,7 +786,7 @@ sendpacket_get_hwaddr_bpf(sendpacket_t *sp)
 
     if (sysctl(mib, 6, buf, &len, NULL, 0) == -1) {
         sendpacket_seterr(sp, "%s(): sysctl(): %s", __func__, strerror(errno));
-        free(buf);
+        safe_free(buf);
         return NULL;
     }
     
@@ -767,13 +801,14 @@ sendpacket_get_hwaddr_bpf(sendpacket_t *sp)
             }
         }
     }
-    free(buf);
+    safe_free(buf);
     return(&sp->ether);
 }
 
 #endif /* HAVE_BPF */
 
-/*
+/**
+ * Get the DLT type of the opened sendpacket
  * Return -1 if we can't figure it out, else return the DLT_ value
  */
 int 
