@@ -1,4 +1,4 @@
-/* $Id: utils.h 1757 2007-03-22 05:38:56Z aturner $ */
+/* $Id: utils.h 1921 2007-10-25 18:18:50Z aturner $ */
 
 /*
  * Copyright (c) 2001-2005 Aaron Turner.
@@ -53,8 +53,17 @@ void *_our_safe_realloc(void *ptr, size_t len, const char *, const int, const ch
 #define safe_strdup(x) _our_safe_strdup(x, __FUNCTION__, __LINE__, __FILE__)
 char *_our_safe_strdup(const char *str, const char *, const int, const char *);
 
+#define safe_free(x) _our_safe_free(x, __FUNCTION__, __LINE__, __FILE__)
+void _our_safe_free(void *ptr, const char *, const int, const char *);
+
 #define MAX_ARGS 128
 
+#ifndef HAVE_INET_ATON
+#define HAVE_INET_ATON
+#define USE_CUSTOM_INET_ATON
+int inet_aton(const char *name, struct in_addr *addr);
+#endif
+
 #endif
 
 /*
