@@ -1,4 +1,4 @@
-/* $Id: tcpbridge.c 1845 2007-04-27 02:48:10Z aturner $ */
+/* $Id: tcpbridge.c 1842 2007-04-26 04:34:11Z aturner $ */
 
 /*
  * Copyright (c) 2004-2005 Aaron Turner.
@@ -154,7 +154,7 @@ init(void)
 
 
 void 
-post_args(__attribute__((unused)) int argc, __attribute__((unused)) char *argv[])
+post_args(_U_ int argc, _U_ char *argv[])
 {
     char ebuf[SENDPACKET_ERRBUF_SIZE];
     char *intname;
