@@ -1,4 +1,4 @@
-/* $Id$ */
+/* $Id: en10mb.c 1921 2007-10-25 18:18:50Z aturner $ */
 
 /*
  * Copyright (c) 2006-2007 Aaron Turner.
@@ -42,7 +42,7 @@
 #include "tcpedit_stub.h"
 #include "../ethernet.h"
 
-static char __attribute__((unused)) dlt_name[] = "en10mb";
+static char _U_ dlt_name[] = "en10mb";
 static char dlt_prefix[] = "enet";
 static u_int16_t dlt_value = DLT_EN10MB;
 
@@ -90,7 +90,8 @@ dlt_en10mb_register(tcpeditdlt_t *ctx)
     plugin->plugin_l2len = dlt_en10mb_l2len;
     plugin->plugin_get_layer3 = dlt_en10mb_get_layer3;
     plugin->plugin_merge_layer3 = dlt_en10mb_merge_layer3;
-
+    plugin->plugin_get_mac = dlt_en10mb_get_mac;
+    
     /* add it to the available plugin list */
     return tcpedit_dlt_addplugin(ctx, plugin);
 }
@@ -144,12 +145,12 @@ dlt_en10mb_cleanup(tcpeditdlt_t *ctx)
         return TCPEDIT_OK;
 
     if (ctx->decoded_extra != NULL) {
-        free(ctx->decoded_extra);
+        safe_free(ctx->decoded_extra);
         ctx->decoded_extra = NULL;
     }
 
     if (plugin->config != NULL) {
-        free(plugin->config);
+        safe_free(plugin->config);
         plugin->config = NULL;
     }
         
@@ -302,7 +303,7 @@ dlt_en10mb_decode(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     extra->vlan = 0;
     
     /* get the L3 protocol type  & L2 len*/
-    switch (eth->ether_type) {
+    switch (ntohs(eth->ether_type)) {
         case ETHERTYPE_VLAN:
             vlan = (struct tcpr_802_1q_hdr *)packet;
             ctx->proto = vlan->vlan_len;
@@ -341,7 +342,6 @@ dlt_en10mb_encode(tcpeditdlt_t *ctx, u_char **packet_ex, int pktlen, tcpr_dir_t
     en10mb_extra_t *extra = NULL;
     
     int newl2len = 0;
-    u_char tmpbuff[MAXPACKET];
 
     assert(ctx);
     assert(packet_ex);
@@ -372,12 +372,8 @@ dlt_en10mb_encode(tcpeditdlt_t *ctx, u_char **packet_ex, int pktlen, tcpr_dir_t
     }
 
     /* Make space for our new L2 header */
-    if (newl2len > ctx->l2len) {
-        memcpy(tmpbuff, packet, pktlen);
-        memcpy(packet + newl2len, (tmpbuff + ctx->l2len), pktlen - ctx->l2len);
-    } else if (newl2len < ctx->l2len) {
+    if (newl2len != ctx->l2len)
         memmove(packet + newl2len, packet + ctx->l2len, pktlen - ctx->l2len);
-    } /* else same size, so do nothing */
 
     /* update the total packet length */
     pktlen += newl2len - ctx->l2len;
@@ -521,14 +517,14 @@ dlt_en10mb_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     assert(pktlen);
     
     eth = (struct tcpr_ethernet_hdr *)packet;
-    switch (eth->ether_type) {
+    switch (ntohs(eth->ether_type)) {
         case ETHERTYPE_VLAN:
             vlan = (struct tcpr_802_1q_hdr *)packet;
-            return ntohs(vlan->vlan_len);
+            return vlan->vlan_len;
             break;
         
         default:
-            return ntohs(eth->ether_type);
+            return eth->ether_type;
             break;
     }
     return TCPEDIT_ERROR;
@@ -570,6 +566,35 @@ dlt_en10mb_merge_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen, u_c
     return tcpedit_dlt_l3data_merge(ctx, packet, pktlen, l3data, l2len);
 }
 
+/*
+ * return a static pointer to the source/destination MAC address
+ * return NULL on error/address doesn't exist
+ */    
+u_char *
+dlt_en10mb_get_mac(tcpeditdlt_t *ctx, tcpeditdlt_mac_type_t mac, const u_char *packet, const int pktlen)
+{
+    assert(ctx);
+    assert(packet);
+    assert(pktlen);
+
+    /* FIXME: return a ptr to the source or dest mac address. */
+    switch(mac) {
+    case SRC_MAC:
+        memcpy(ctx->srcmac, &packet[6], ETHER_ADDR_LEN);
+        return(ctx->srcmac);
+        break;
+        
+    case DST_MAC:
+        memcpy(ctx->dstmac, packet, ETHER_ADDR_LEN);
+        return(ctx->dstmac);
+        break;
+        
+    default:
+        errx(1, "Invalid tcpeditdlt_mac_type_t: %d", mac);
+    }
+    return(NULL);
+}
+
 /* 
  * return the length of the L2 header of the current packet
  */
@@ -583,7 +608,7 @@ dlt_en10mb_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     assert(pktlen);
     
     eth = (struct tcpr_ethernet_hdr *)packet;
-    switch (eth->ether_type) {
+    switch (ntohs(eth->ether_type)) {
         case ETHERTYPE_VLAN:
             return 18;
             break;
