@@ -1,4 +1,4 @@
-/* $Id:$ */
+/* $Id: hdlc.c 1921 2007-10-25 18:18:50Z aturner $ */
 
 /*
  * Copyright (c) 2006-2007 Aaron Turner.
@@ -41,7 +41,7 @@
 #include "tcpr.h"
 
 static char dlt_name[] = "hdlc";
-static char __attribute__((unused)) dlt_prefix[] = "hdlc";
+static char _U_ dlt_prefix[] = "hdlc";
 static u_int16_t dlt_value = DLT_C_HDLC;
 
 /*
@@ -88,7 +88,8 @@ dlt_hdlc_register(tcpeditdlt_t *ctx)
     plugin->plugin_l2len = dlt_hdlc_l2len;
     plugin->plugin_get_layer3 = dlt_hdlc_get_layer3;
     plugin->plugin_merge_layer3 = dlt_hdlc_merge_layer3;
-
+    plugin->plugin_get_mac = dlt_hdlc_get_mac;
+    
     /* add it to the available plugin list */
     return tcpedit_dlt_addplugin(ctx, plugin);
 }
@@ -145,12 +146,12 @@ dlt_hdlc_cleanup(tcpeditdlt_t *ctx)
     }
 
     if (ctx->decoded_extra != NULL) {
-        free(ctx->decoded_extra);
+        safe_free(ctx->decoded_extra);
         ctx->decoded_extra = NULL;
     }
         
     if (plugin->config != NULL) {
-        free(plugin->config);
+        safe_free(plugin->config);
         plugin->config = NULL;
     }
 
@@ -220,7 +221,7 @@ dlt_hdlc_decode(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
  * Returns: total packet len or TCPEDIT_ERROR
  */
 int 
-dlt_hdlc_encode(tcpeditdlt_t *ctx, u_char **packet_ex, int pktlen, __attribute__((unused)) tcpr_dir_t dir)
+dlt_hdlc_encode(tcpeditdlt_t *ctx, u_char **packet_ex, int pktlen, _U_ tcpr_dir_t dir)
 {
     cisco_hdlc_t *hdlc;
     hdlc_config_t *config = NULL;
@@ -295,7 +296,7 @@ dlt_hdlc_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     
     hdlc = (cisco_hdlc_t *)packet;
     
-    return ntohs(hdlc->protocol);
+    return hdlc->protocol;
 }
 
 /*
@@ -352,6 +353,33 @@ dlt_hdlc_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     return 4;
 }
 
+/*
+ * return a static pointer to the source/destination MAC address
+ * return NULL on error/address doesn't exist
+ */    
+u_char *
+dlt_hdlc_get_mac(tcpeditdlt_t *ctx, tcpeditdlt_mac_type_t mac, const u_char *packet, const int pktlen)
+{
+    assert(ctx);
+    assert(packet);
+    assert(pktlen);
+
+    /* FIXME: return a ptr to the source or dest mac address. */
+    switch(mac) {
+    case SRC_MAC:
+        return(NULL);
+        break;
+        
+    case DST_MAC:
+        memcpy(ctx->dstmac, packet, 2);
+        return(ctx->dstmac);
+        break;
+        
+    default:
+        errx(1, "Invalid tcpeditdlt_mac_type_t: %d", mac);
+    }
+    return(NULL);
+}
 
 tcpeditdlt_l2addr_type_t 
 dlt_hdlc_l2addr_type(void)
