@@ -1,4 +1,4 @@
-/* $Id:$ */
+/* $Id: linuxsll.c 1921 2007-10-25 18:18:50Z aturner $ */
 
 /*
  * Copyright (c) 2006-2007 Aaron Turner.
@@ -41,7 +41,7 @@
 #include "tcpr.h"
 
 static char dlt_name[] = "linuxsll";
-static char __attribute__((unused)) dlt_prefix[] = "linuxsll";
+static char _U_ dlt_prefix[] = "linuxsll";
 static u_int16_t dlt_value = DLT_LINUX_SLL;
 
 /*
@@ -89,7 +89,8 @@ dlt_linuxsll_register(tcpeditdlt_t *ctx)
     plugin->plugin_l2len = dlt_linuxsll_l2len;
     plugin->plugin_get_layer3 = dlt_linuxsll_get_layer3;
     plugin->plugin_merge_layer3 = dlt_linuxsll_merge_layer3;
-
+    plugin->plugin_get_mac = dlt_linuxsll_get_mac;
+    
     /* add it to the available plugin list */
     return tcpedit_dlt_addplugin(ctx, plugin);
 }
@@ -144,12 +145,12 @@ dlt_linuxsll_cleanup(tcpeditdlt_t *ctx)
 
     /* FIXME: make this function do something if necessary */
     if (ctx->decoded_extra != NULL) {
-        free(ctx->decoded_extra);
+        safe_free(ctx->decoded_extra);
         ctx->decoded_extra = NULL;
     }
         
     if (plugin->config != NULL) {
-        free(plugin->config);
+        safe_free(plugin->config);
         plugin->config = NULL;
     }
 
@@ -209,7 +210,7 @@ dlt_linuxsll_decode(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
  * Returns: total packet len or TCPEDIT_ERROR
  */
 int 
-dlt_linuxsll_encode(tcpeditdlt_t *ctx, u_char **packet_ex, int pktlen, __attribute__((unused)) tcpr_dir_t dir)
+dlt_linuxsll_encode(tcpeditdlt_t *ctx, u_char **packet_ex, int pktlen, _U_ tcpr_dir_t dir)
 {
     u_char *packet;
     assert(ctx);
@@ -236,7 +237,7 @@ dlt_linuxsll_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
 
     linux_sll = (linux_sll_header_t *)packet;
     
-    return ntohs(linux_sll->proto);
+    return linux_sll->proto;
 }
 
 /*
@@ -290,6 +291,33 @@ dlt_linuxsll_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     return sizeof(linux_sll_header_t);
 }
 
+/*
+ * return a static pointer to the source/destination MAC address
+ * return NULL on error/address doesn't exist
+ */    
+u_char *
+dlt_linuxsll_get_mac(tcpeditdlt_t *ctx, tcpeditdlt_mac_type_t mac, const u_char *packet, const int pktlen)
+{
+    assert(ctx);
+    assert(packet);
+    assert(pktlen);
+
+    /* FIXME: return a ptr to the source or dest mac address. */
+    switch(mac) {
+    case SRC_MAC:
+        memcpy(ctx->srcmac, &packet[6], 8); /* linuxssl defines the src mac field to be 8 bytes, not 6 */
+        return(ctx->srcmac);
+        break;
+        
+    case DST_MAC:
+        return(NULL);
+        break;
+        
+    default:
+        errx(1, "Invalid tcpeditdlt_mac_type_t: %d", mac);
+    }
+    return(NULL);
+}
 
 tcpeditdlt_l2addr_type_t 
 dlt_linuxsll_l2addr_type(void)
