@@ -1,4 +1,4 @@
-/* $Id:$ */
+/* $Id: null.c 1921 2007-10-25 18:18:50Z aturner $ */
 
 /*
  * Copyright (c) 2006-2007 Aaron Turner.
@@ -43,7 +43,7 @@
 #include <sys/socket.h> // PF_* values
 
 static char dlt_name[] = "null";
-static char __attribute__((unused)) dlt_prefix[] = "null";
+static char _U_ dlt_prefix[] = "null";
 static u_int16_t dlt_value = DLT_NULL;
 
 /*
@@ -162,12 +162,12 @@ dlt_null_cleanup(tcpeditdlt_t *ctx)
     }
 
     if (ctx->decoded_extra != NULL) {
-        free(ctx->decoded_extra);
+        safe_free(ctx->decoded_extra);
         ctx->decoded_extra = NULL;
     }
         
     if (plugin->config != NULL) {
-        free(plugin->config);
+        safe_free(plugin->config);
         plugin->config = NULL;
     }
 
@@ -221,7 +221,7 @@ dlt_null_decode(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
  * Returns: total packet len or TCPEDIT_ERROR
  */
 int 
-dlt_null_encode(tcpeditdlt_t *ctx, u_char **packet_ex, int pktlen, __attribute__((unused)) tcpr_dir_t dir)
+dlt_null_encode(tcpeditdlt_t *ctx, u_char **packet_ex, int pktlen, _U_ tcpr_dir_t dir)
 {
     u_char *packet;
     assert(ctx);
@@ -257,7 +257,7 @@ dlt_null_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
         return TCPEDIT_ERROR;
     }
     
-    return protocol; 
+    return htons(protocol);
 }
 
 /*
@@ -312,6 +312,20 @@ dlt_null_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     return 4;
 }
 
+/*
+ * return a static pointer to the source/destination MAC address
+ * return NULL on error/address doesn't exist
+ */    
+u_char *
+dlt_null_get_mac(tcpeditdlt_t *ctx, _U_ tcpeditdlt_mac_type_t mac, const u_char *packet, const int pktlen)
+{
+    assert(ctx);
+    assert(packet);
+    assert(pktlen);
+
+    return(NULL);
+
+}
 
 tcpeditdlt_l2addr_type_t 
 dlt_null_l2addr_type(void)
