@@ -1,4 +1,4 @@
-/* $Id:$ */
+/* $Id: raw.c 1921 2007-10-25 18:18:50Z aturner $ */
 
 /*
  * Copyright (c) 2006-2007 Aaron Turner.
@@ -42,7 +42,7 @@
 
 /* FIXME: edit these variables to taste */
 static char dlt_name[] = "raw";
-static char __attribute__((unused)) dlt_prefix[] = "raw";
+static char _U_ dlt_prefix[] = "raw";
 static u_int16_t dlt_value = DLT_RAW;
 
 /*
@@ -92,7 +92,8 @@ dlt_raw_register(tcpeditdlt_t *ctx)
     plugin->plugin_l2len = dlt_raw_l2len;
     plugin->plugin_get_layer3 = dlt_raw_get_layer3;
     plugin->plugin_merge_layer3 = dlt_raw_merge_layer3;
-
+    plugin->plugin_get_mac = dlt_raw_get_mac;
+    
     /* add it to the available plugin list */
     return tcpedit_dlt_addplugin(ctx, plugin);
 }
@@ -147,12 +148,12 @@ dlt_raw_cleanup(tcpeditdlt_t *ctx)
 
     /* FIXME: make this function do something if necessary */
     if (ctx->decoded_extra != NULL) {
-        free(ctx->decoded_extra);
+        safe_free(ctx->decoded_extra);
         ctx->decoded_extra = NULL;
     }
         
     if (plugin->config != NULL) {
-        free(plugin->config);
+        safe_free(plugin->config);
         plugin->config = NULL;
     }
 
@@ -206,7 +207,7 @@ dlt_raw_decode(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
  * Returns: total packet len or TCPEDIT_ERROR
  */
 int 
-dlt_raw_encode(tcpeditdlt_t *ctx, u_char **packet_ex, int pktlen, __attribute__((unused))tcpr_dir_t dir)
+dlt_raw_encode(tcpeditdlt_t *ctx, u_char **packet_ex, int pktlen, _U_ tcpr_dir_t dir)
 {
     u_char *packet;
     assert(ctx);
@@ -242,7 +243,7 @@ dlt_raw_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
         return TCPEDIT_ERROR;
     }
     
-    return protocol; 
+    return htons(protocol);
 }
 
 /*
@@ -292,6 +293,19 @@ dlt_raw_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     return 0;
 }
 
+/*
+ * return a static pointer to the source/destination MAC address
+ * return NULL on error/address doesn't exist
+ */    
+u_char *
+dlt_raw_get_mac(tcpeditdlt_t *ctx, _U_ tcpeditdlt_mac_type_t mac, const u_char *packet, const int pktlen)
+{
+    assert(ctx);
+    assert(packet);
+    assert(pktlen);
+
+    return(NULL);
+}
 
 tcpeditdlt_l2addr_type_t 
 dlt_raw_l2addr_type(void)
