@@ -1,4 +1,4 @@
-/* $Id: tcpedit.c 1855 2007-05-02 04:54:38Z aturner $ */
+/* $Id: tcpedit.c 1921 2007-10-25 18:18:50Z aturner $ */
 
 /*
  * Copyright (c) 2001-2007 Aaron Turner.
@@ -56,12 +56,15 @@
 
 tOptDesc *const tcpedit_tcpedit_optDesc_p;
 
-/* 
+/**
+ * \brief Edit the given packet
+ *
  * Processs a given packet and edit the pkthdr/pktdata structures
  * according to the rules in tcpedit
- * Returns: -1 on error
- *           0 on no change
- *           1 on change
+ * Returns: TCPEDIT_ERROR on error
+ *          TCPEDIT_SOFT_ERROR on remove packet
+ *          0 on no change
+ *          1 on change
  */
 int
 tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
@@ -80,24 +83,24 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     assert(*pktdata);
     assert(tcpedit->validated);
 
- 
+
     tcpedit->runtime.packetnum++;
     dbgx(2, "packet " COUNTER_SPEC " caplen %d", 
             tcpedit->runtime.packetnum, (*pkthdr)->caplen);
-            
-   /* 
-     * if we are padding out the packet, we need to move the packet 
-     * data to a different buffer because the incoming **pktdata buffer
-     * most likely isn't big enough for the extra padding
+
+    /* 
+     * copy over our packet data.  Necessary because many options like padding or 
+     * adding VLAN tags makes the packet size increase and we'll get segfaults 
+     * later on
      */
-    if (HAVE_OPT(FIXLEN) && strcmp(OPT_ARG(FIXLEN), "pad") == 0) {
-        /* allocate our buffer the first time */
-        if (packet == NULL)
-            packet = safe_malloc(MAX_SNAPLEN);
-    
-        memcpy(packet, *pktdata, (*pkthdr)->caplen);
-        *pktdata = packet;
-    }
+            
+    /* allocate our buffer the first time */
+    if (packet == NULL)
+        packet = safe_malloc(MAX_SNAPLEN);
+
+    /* and copy over the packet data... */
+    memcpy(packet, *pktdata, (*pkthdr)->caplen);
+    *pktdata = packet;
     
     /*
      * remove the Ethernet FCS (checksum)?
@@ -112,6 +115,12 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     if ((pktlen = tcpedit_dlt_process(tcpedit->dlt_ctx, *pktdata, (*pkthdr)->caplen, direction)) == TCPEDIT_ERROR)
         errx(1, "%s", tcpedit_geterr(tcpedit));
 
+    /* unable to edit packet, most likely 802.11 management frame */
+    if (pktlen == TCPEDIT_SOFT_ERROR) {
+        dbgx(1, "%s", tcpedit_geterr(tcpedit));
+        return TCPEDIT_SOFT_ERROR;
+    }
+
     /* update our packet lengths (real/captured) based on L2 length changes */
     lendiff = pktlen - (*pkthdr)->caplen;
     (*pkthdr)->caplen += lendiff;
@@ -122,10 +131,10 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     l2len = tcpedit_dlt_l2len(tcpedit->dlt_ctx, dlt, *pktdata, (*pkthdr)->caplen);
 
     /* does packet have an IP header?  if so set our pointer to it */
-    if (l2proto == ETHERTYPE_IP) {
+    if (l2proto == htons(ETHERTYPE_IP)) {
         ip_hdr = (ipv4_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, dlt, *pktdata, (*pkthdr)->caplen);
         if (ip_hdr == NULL) {
-            return -1;
+            return TCPEDIT_ERROR;
         }        
         dbg(3, "Packet has an IPv4 header...");
     } else {
@@ -139,33 +148,33 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         /* IP packets */
         if (ip_hdr != NULL) {
             if ((retval = rewrite_ipv4l3(tcpedit, ip_hdr, direction)) < 0)
-                return -1;
+                return TCPEDIT_ERROR;
             needtorecalc += retval;
         }
 
         /* ARP packets */
-        else if (l2proto == ETHERTYPE_ARP) {
+        else if (l2proto == htons(ETHERTYPE_ARP)) {
             arp_hdr = (arp_hdr_t *)(&(*pktdata)[l2len]);
             /* unlike, rewrite_ipl3, we don't care if the packet changed
              * because we never need to recalc the checksums for an ARP
              * packet.  So ignore the return value
              */
             if (rewrite_iparp(tcpedit, arp_hdr, direction) < 0)
-                return -1;
+                return TCPEDIT_ERROR;
         }
     }
 
     /* rewrite ports */
     if (tcpedit->portmap != NULL && (ip_hdr != NULL)) {
         if ((retval = rewrite_ports(tcpedit, &ip_hdr)) < 0)
-            return -1;
+            return TCPEDIT_ERROR;
         needtorecalc += retval;
     }
 
     /* Untruncate packet? Only for IP packets */
     if ((tcpedit->fixlen) && (ip_hdr != NULL)) {
         if ((retval = untrunc_packet(tcpedit, *pkthdr, *pktdata, ip_hdr)) < 0)
-            return -1;
+            return TCPEDIT_ERROR;
         needtorecalc += retval;
     }
 
@@ -175,17 +184,17 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         if (ip_hdr != NULL) {
             if ((retval = randomize_ipv4(tcpedit, *pkthdr, *pktdata, 
                     ip_hdr)) < 0)
-                return -1;
+                return TCPEDIT_ERROR;
             needtorecalc += retval;
         } else {
             if (direction == TCPR_DIR_C2S) {
                 if (randomize_iparp(tcpedit, *pkthdr, *pktdata, 
                         tcpedit->runtime.dlt1) < 0)
-                    return -1;
+                    return TCPEDIT_ERROR;
             } else {
                 if (randomize_iparp(tcpedit, *pkthdr, *pktdata, 
                         tcpedit->runtime.dlt2) < 0)
-                    return -1;
+                    return TCPEDIT_ERROR;
             }
         }
     }
@@ -208,7 +217,7 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     return retval;
 }
 
-/*
+/**
  * initializes the tcpedit library.  returns 0 on success, -1 on error.
  */
 int
@@ -238,8 +247,8 @@ tcpedit_init(tcpedit_t **tcpedit_ex, int dlt)
     return TCPEDIT_OK;
 }
 
-/*
- * return the output DLT 
+/**
+ * return the output DLT type
  */
 int
 tcpedit_get_output_dlt(tcpedit_t *tcpedit)
@@ -248,7 +257,9 @@ tcpedit_get_output_dlt(tcpedit_t *tcpedit)
     return tcpedit_dlt_output_dlt(tcpedit->dlt_ctx);
 }
 
-/*
+/**
+ * \brief tcpedit option validator.  Call after tcpedit_init()
+ *
  * Validates that given the current state of tcpedit that the given
  * pcap source and destination (based on DLT) can be properly rewritten
  * return 0 on sucess
@@ -267,9 +278,9 @@ tcpedit_validate(tcpedit_t *tcpedit)
     return 0;
 }
 
-/*
+/**
  * return the error string when a tcpedit() function returns
- * an error 
+ * TCPEDIT_ERROR
  */
 char *
 tcpedit_geterr(tcpedit_t *tcpedit)
@@ -280,8 +291,12 @@ tcpedit_geterr(tcpedit_t *tcpedit)
 
 }
 
-/*
- * used to set the error string when there is an error
+/**
+ * \brief Internal function to set the tcpedit error string
+ *
+ * Used to set the error string when there is an error, result is retrieved
+ * using tcpedit_geterr().  You shouldn't ever actually call this, but use
+ * tcpedit_seterr() which is a macro wrapping this instead.
  */
 void
 __tcpedit_seterr(tcpedit_t *tcpedit, const char *func, const int line, const char *file, const char *fmt, ...)
@@ -293,7 +308,6 @@ __tcpedit_seterr(tcpedit_t *tcpedit, const char *func, const int line, const cha
 
     va_start(ap, fmt);
     if (fmt != NULL) {
-        dbgx(1, fmt, ap);
         (void)vsnprintf(errormsg, 
               (TCPEDIT_ERRSTR_LEN - 1), fmt, ap);
     }
@@ -304,9 +318,9 @@ __tcpedit_seterr(tcpedit_t *tcpedit, const char *func, const int line, const cha
         file, func, line, errormsg);
 }
 
-/*
+/**
  * return the warning string when a tcpedit() function returns
- * a warning
+ * TCPEDIT_WARN
  */
 char *
 tcpedit_getwarn(tcpedit_t *tcpedit)
@@ -316,7 +330,7 @@ tcpedit_getwarn(tcpedit_t *tcpedit)
     return tcpedit->runtime.warnstr;
 }
 
-/*
+/**
  * used to set the warning string when there is an warning
  */
 void
@@ -336,11 +350,15 @@ tcpedit_setwarn(tcpedit_t *tcpedit, const char *fmt, ...)
         
 }
 
-/*
+/**
+ * \brief Checks the given error code and does the right thing
+ * 
  * Generic function which checks the TCPEDIT_* error code
  * and always returns OK or ERROR.  For warnings, prints the 
  * warning message and returns OK.  For any other value, fails with
  * an assert.
+ *
+ * prefix is a string prepended to the error/warning
  */
 int
 tcpedit_checkerror(tcpedit_t *tcpedit, const int rcode, const char *prefix) {
@@ -351,7 +369,14 @@ tcpedit_checkerror(tcpedit_t *tcpedit, const int rcode, const char *prefix) {
         case TCPEDIT_ERROR:
             return rcode;
             break;
-            
+        
+        case TCPEDIT_SOFT_ERROR:
+            if (prefix != NULL) {
+                fprintf(stderr, "Error %s: %s\n", prefix, tcpedit_geterr(tcpedit));
+            } else {
+                fprintf(stderr, "Error: %s\n", tcpedit_geterr(tcpedit));
+            }            
+            break;
         case TCPEDIT_WARN:
             if (prefix != NULL) {
                 fprintf(stderr, "Warning %s: %s\n", prefix, tcpedit_getwarn(tcpedit));
@@ -368,8 +393,10 @@ tcpedit_checkerror(tcpedit_t *tcpedit, const int rcode, const char *prefix) {
     return TCPEDIT_ERROR;
 }
 
-/*
- * Cleans up after ourselves.  Return 0 on success.
+/**
+ * \brief Cleans up after ourselves.  Return 0 on success. 
+ * 
+ * Clean up after ourselves, but does not actually free the ptr.
  */
 int
 tcpedit_close(tcpedit_t *tcpedit)
@@ -382,14 +409,16 @@ tcpedit_close(tcpedit_t *tcpedit)
 
     /* free buffer if required */
 #ifdef FORCE_ALIGN
-    free(tcpedit->runtime.l3buff);
+    safe_free(tcpedit->runtime.l3buff);
 #endif
 
     return 0;
 }
 
-
-u_char *
+/**
+ * Return a ptr to the Layer 3 data.  Returns TCPEDIT_ERROR on error
+ */
+const u_char *
 tcpedit_l3data(tcpedit_t *tcpedit, tcpedit_coder_t code, u_char *packet, const int pktlen)
 {
     u_char *result = NULL;
@@ -401,6 +430,24 @@ tcpedit_l3data(tcpedit_t *tcpedit, tcpedit_coder_t code, u_char *packet, const i
     return result;
 }
 
+/**
+ * return the length of the layer 2 header.  Returns TCPEDIT_ERROR on error
+ */
+int 
+tcpedit_l2len(tcpedit_t *tcpedit, tcpedit_coder_t code, u_char *packet, const int pktlen)
+{
+    int result = 0;
+    if (code == BEFORE_PROCESS) {
+        result = tcpedit_dlt_l2len(tcpedit->dlt_ctx, tcpedit->dlt_ctx->decoder->dlt, packet, pktlen);
+    } else {
+        result = tcpedit_dlt_l2len(tcpedit->dlt_ctx, tcpedit->dlt_ctx->encoder->dlt, packet, pktlen);
+    }
+    return result;
+}
+
+/**
+ * Returns the layer 3 type, often encoded as the layer2.proto field
+ */
 int 
 tcpedit_l3proto(tcpedit_t *tcpedit, tcpedit_coder_t code, const u_char *packet, const int pktlen)
 {
@@ -413,12 +460,23 @@ tcpedit_l3proto(tcpedit_t *tcpedit, tcpedit_coder_t code, const u_char *packet,
     return result;
 }
 
-
 /*
- Local Variables:
- mode:c
- indent-tabs-mode:nil
- c-basic-offset:4
- End:
-*/
+u_char *
+tcpedit_srcmac(tcpedit_t *tcpedit, tcpedit_coder_t code, u_char *packet, const int pktlen)
+{
+   
+}
 
+u_char *
+tcpedit_dstmac(tcpedit_t *tcpedit, tcpedit_coder_t code, u_char *packet, const int pktlen)
+{
+    
+}
+
+int 
+tcpedit_maclen(tcpedit_t *tcpedit, tcpedit_coder_t code)
+{
+    
+}
+
+*/
\ No newline at end of file
