@@ -1,4 +1,4 @@
-/* $Id: tcprewrite.c 1820 2007-04-19 06:31:03Z aturner $ */
+/* $Id: tcprewrite.c 1898 2007-08-25 05:10:51Z aturner $ */
 
 /*
  * Copyright (c) 2004-2007 Aaron Turner.
@@ -72,7 +72,8 @@ void post_args(int argc, char *argv[]);
 void verify_input_pcap(pcap_t *pcap);
 int rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout);
 
-int main(int argc, char *argv[])
+int 
+main(int argc, char *argv[])
 {
     int optct, rcode;
     pcap_t *dlt_pcap;
@@ -157,9 +158,11 @@ tcprewrite_init(void)
         warnx("Unable to set STDERR to non-blocking: %s", strerror(errno));
 }
 
-
+/**
+ * post AutoGen argument processing
+ */
 void 
-post_args(__attribute__((unused)) int argc, __attribute__((unused)) char *argv[])
+post_args(_U_ int argc, _U_ char *argv[])
 {
     char ebuf[PCAP_ERRBUF_SIZE];
      
@@ -186,6 +189,9 @@ post_args(__attribute__((unused)) int argc, __attribute__((unused)) char *argv[]
         errx(1, "Unable to open input pcap file: %s", ebuf);
 }
 
+/** 
+ * Main loop to rewrite packets
+ */
 int
 rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout)
 {
@@ -194,6 +200,7 @@ rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout)
     const u_char *pktdata = NULL;               /* packet from libpcap */
     u_char **packet = NULL;                     /* packet from tcpedit */
     COUNTER packetnum = 0;
+    int rcode;
 
     pkthdr_ptr = &pkthdr;
 
@@ -225,8 +232,12 @@ rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout)
             goto WRITE_PACKET; /* still need to write it so cache stays in sync */
 
         packet = &pktdata;
-        if (tcpedit_packet(tcpedit, &pkthdr_ptr, packet, cache_result) == -1) {
+        if ((rcode = tcpedit_packet(tcpedit, &pkthdr_ptr, packet, cache_result)) == TCPEDIT_ERROR) {
             return -1;
+        } else if ((rcode == TCPEDIT_SOFT_ERROR) && HAVE_OPT(SKIP_SOFT_ERRORS)) {
+            /* don't write packet */
+            dbgx(1, "Packet " COUNTER_SPEC " is suppressed from being written due to soft errors", packetnum);
+            continue;
         }
 
 
