@@ -1,4 +1,4 @@
-/* $Id: tree.c 1801 2007-04-14 22:08:33Z aturner $ */
+/* $Id: tree.c 1900 2007-08-26 19:47:53Z aturner $ */
 
 /*
  * Copyright (c) 2001-2007 Aaron Turner.
@@ -61,7 +61,7 @@ static int tree_checkincidr(tcpr_data_tree_t *, tcpr_buildcidr_t *);
 RB_PROTOTYPE(tcpr_data_tree_s, tcpr_tree_s, node, tree_comp)
 RB_GENERATE(tcpr_data_tree_s, tcpr_tree_s, node, tree_comp)
 
-/*
+/**
  * used with rbwalk to walk a tree and generate cidr_t * cidrdata.
  * is smart enough to prevent dupes.  void * arg is cast to bulidcidr_t
  */
@@ -101,10 +101,9 @@ tree_buildcidr(tcpr_data_tree_t *treeroot, tcpr_buildcidr_t * bcdata)
 }
 
 
-/*
+/**
  * uses rbwalk to check to see if a given ip address of a given type in the
  * tree is inside any of the cidrdata
- *
  */
 static int
 tree_checkincidr(tcpr_data_tree_t *treeroot, tcpr_buildcidr_t * bcdata)
@@ -123,22 +122,21 @@ tree_checkincidr(tcpr_data_tree_t *treeroot, tcpr_buildcidr_t * bcdata)
          * in cases of leaves and last visit add to cidrdata if
          * necessary
          */
-        if (check_ip_cidr(options.cidrdata, node->ip)) {    /* if we exist, abort */
+        if (check_ip_cidr(options.cidrdata, node->ip))    /* if we exist, abort */
             return 1;
-        }
+
     }
     return 0;
 }
 
-/*
+/**
  * processes the tree using rbwalk / tree2cidr to generate a CIDR
  * used for 2nd pass, router mode
  *
  * returns > 0 for success (the mask len), 0 for fail
  */
-
 int
-process_tree()
+process_tree(void)
 {
     int mymask = 0;
     tcpr_buildcidr_t *bcdata;
@@ -165,6 +163,7 @@ process_tree()
         bcdata->type = DIR_CLIENT;
 
         if (! tree_checkincidr(&treeroot, bcdata)) { /* didn't find any clients in cidrdata */
+            safe_free(bcdata);
             return (mymask);    /* success! */
         }
         else {
@@ -173,7 +172,10 @@ process_tree()
         }
     }
 
+    safe_free(bcdata);
     /* we failed to find a vaild cidr list */
+    notice("Unable to determine any IP addresses as a clients.");
+    notice("Perhaps you should change the --ratio, --minmask/maxmask settings, or try another mode?");
     return (0);
 }
 
@@ -190,7 +192,7 @@ tcpr_tree_to_cidr(const int masklen, const int type)
 }
  */
 
-/*
+/**
  * Checks to see if an IP is client or server by finding it in the tree
  * returns TCPR_DIR_C2S or TCPR_DIR_S2C or -1 on error
  * if mode = UNKNOWN, then abort on unknowns
@@ -213,14 +215,19 @@ check_ip_tree(const int mode, const unsigned long ip)
              get_addr2name4(ip, RESOLVE), ip);
 
 #ifdef DEBUG
-    if (node->type == DIR_SERVER) {
-        dbgx(1, "Server: %s", get_addr2name4(ip, RESOLVE));
-    }
-    else if (node->type == DIR_CLIENT) {
-        dbgx(1, "Client: %s", get_addr2name4(ip, RESOLVE));
-    }
-    else {
-        dbgx(1, "Unknown: %s", get_addr2name4(ip, RESOLVE));
+    switch (node->type) {
+    case DIR_SERVER:
+        dbgx(1, "DIR_SERVER: %s", get_addr2name4(ip, RESOLVE));
+        break;
+    case DIR_CLIENT:
+        dbgx(1, "DIR_CLIENT: %s", get_addr2name4(ip, RESOLVE));
+        break;
+    case DIR_UNKNOWN:
+        dbgx(1, "DIR_UNKNOWN: %s", get_addr2name4(ip, RESOLVE));
+        break;
+    case DIR_ANY:
+        dbgx(1, "DIR_ANY: %s", get_addr2name4(ip, RESOLVE));
+        break;
     }
 #endif
 
@@ -230,32 +237,101 @@ check_ip_tree(const int mode, const unsigned long ip)
 
     /* return node type if we found the node, else return the default (mode) */
     if (node != NULL) {
-        if (node->type == DIR_SERVER) {
+        switch (node->type) {
+        case DIR_SERVER:
             return TCPR_DIR_C2S;
-        } else if (node->type == DIR_CLIENT) {
+            break;
+        case DIR_CLIENT:
             return TCPR_DIR_S2C;
+            break;
+        case DIR_UNKNOWN:
+        case DIR_ANY:
+            /* use our current mode to determine return code */
+            goto return_unknown; 
+        default:
+            errx(1, "Node for %s has invalid type: %d", get_addr2name4(ip, RESOLVE), node->type);
         }
     }
-    else {
-        if (mode == DIR_SERVER) {
-            return TCPR_DIR_C2S;
-        } else if (mode == DIR_CLIENT) {
-            return TCPR_DIR_S2C;
-        }
+    
+    return_unknown:
+    switch (mode) {
+    case DIR_SERVER:
+        return TCPR_DIR_C2S;
+        break;
+    case DIR_CLIENT:
+        return TCPR_DIR_S2C;
+        break;
+    default:
+        return -1;
     }
-    err(1, "Uh, we shouldn't of gotten here.");
-    return(-1);
 }
 
-/*
- * adds an entry to the tree (phase 1 of auto mode)
+/**
+ * Parses the IP header of the given packet (data) to get the SRC/DST IP 
+ * addresses.  If the SRC IP doesn't exist in the TREE, we add it as a
+ * client, if the DST IP doesn't exist in the TREE, we add it as a server
  */
+void
+add_tree_first(const u_char *data)
+{
+    tcpr_tree_t *newnode = NULL, *findnode;
+    eth_hdr_t *eth_hdr = NULL;
+    ipv4_hdr_t ip_hdr;
+    
+    assert(data);
+    /* 
+     * first add/find the source IP/client 
+     */
+    newnode = new_tree();
 
+    eth_hdr = (eth_hdr_t *) (data);
+    /* prevent issues with byte alignment, must memcpy */
+    memcpy(&ip_hdr, (data + TCPR_ETH_H), TCPR_IPV4_H);
+
+    /* copy over the source ip, and values to gurantee this a client */
+    newnode->ip = ip_hdr.ip_src.s_addr;
+    newnode->type = DIR_CLIENT;
+    newnode->client_cnt = 1000;
+    findnode = RB_FIND(tcpr_data_tree_s, &treeroot, newnode);
+    
+    /* if we didn't find it, add it to the tree, else free it */
+    if (findnode == NULL) {
+        RB_INSERT(tcpr_data_tree_s, &treeroot, newnode);
+    } else {
+        safe_free(newnode);
+    }
+    
+    /*
+     * now add/find the destination IP/server
+     */
+    newnode = new_tree();
+    eth_hdr = (eth_hdr_t *) (data);
+    memcpy(&ip_hdr, (data + TCPR_ETH_H), TCPR_IPV4_H);
+
+    newnode->ip = ip_hdr.ip_dst.s_addr;
+    newnode->type = DIR_SERVER;
+    newnode->server_cnt = 1000;
+    findnode = RB_FIND(tcpr_data_tree_s, &treeroot, newnode);
+
+    if (findnode == NULL) {
+        RB_INSERT(tcpr_data_tree_s, &treeroot, newnode);
+    } else {
+        safe_free(newnode);
+    }
+}
+
+/**
+ * adds an entry to the tree (phase 1 of auto mode).  We add each host
+ * to the tree if it doesn't yet exist.  We go through and track:
+ * - number of times each host acts as a client or server
+ * - the way the host acted the first time we saw it (client or server)
+ */
 void
 add_tree(const unsigned long ip, const u_char * data)
 {
     tcpr_tree_t *node = NULL, *newnode = NULL;
-
+    assert(data);
+    
     newnode = packet2tree(data);
 
     assert(ip == newnode->ip);
@@ -297,8 +373,9 @@ add_tree(const unsigned long ip, const u_char * data)
             /* temp debug code */
             node->client_cnt++;
         }
+                    
         /* didn't insert it, so free it */
-        free(newnode);
+        safe_free(newnode);
     }
 
     dbg(2, "------- START NEXT -------");
@@ -306,10 +383,9 @@ add_tree(const unsigned long ip, const u_char * data)
 }
 
 
-/*
- * calculates wether an IP is a client, server, or unknown for each node in the tree
+/**
+ * calculates wether each node in the tree is a client, server, or unknown for each node in the tree
  */
-
 void
 tree_calculate(tcpr_data_tree_t *treeroot)
 {
@@ -340,7 +416,7 @@ tree_calculate(tcpr_data_tree_t *treeroot)
     }
 }
 
-/*
+/**
  * tree_comp(), called by rbsearch compares two treees and returns:
  * 1  = first > second
  * -1 = first < second
@@ -371,10 +447,9 @@ tree_comp(tcpr_tree_t *t1, tcpr_tree_t *t2)
 
 }
 
-/*
+/**
  * creates a new TREE * with reasonable defaults
  */
-
 static tcpr_tree_t *
 new_tree()
 {
@@ -392,13 +467,12 @@ new_tree()
 }
 
 
-/*
+/**
  * returns a struct of TREE * from a packet header
  * and sets the type to be SERVER or CLIENT or UNKNOWN
  * if it's an undefined packet, we return -1 for the type
  * the u_char * data should be the data that is passed by pcap_dispatch()
  */
-
 tcpr_tree_t *
 packet2tree(const u_char * data)
 {
@@ -540,15 +614,13 @@ packet2tree(const u_char * data)
 
     }
 
-
     return (node);
 }
 
 
-/*
+/**
  * prints out a node of the tree to stderr
  */
-
 static char *
 tree_printnode(const char *name, const tcpr_tree_t *node)
 {
@@ -574,10 +646,9 @@ tree_printnode(const char *name, const tcpr_tree_t *node)
     return (tree_print_buff);
 }
 
-/*
+/**
  * prints out the entire tree
  */
-
 static char *
 tree_print(tcpr_data_tree_t *treeroot)
 {
